/* Any copyright is dedicated to the Public Domain.
  http://creativecommons.org/publicdomain/zero/1.0/ */

/*
 * THIS FILE IS AUTOGENERATED. DO NOT MODIFY BY HAND.
 *
 * More info in https://firefox-source-docs.mozilla.org/devtools/tests/js-object-tests.html
 */

export default [
  // undefined
  {
    type: "undefined",
  },

  // null
  {
    type: "null",
  },

  // true
  true,

  // false
  false,

  // NaN
  {
    type: "NaN",
  },

  // "abc"
  "abc",

  // "鼬ú"
  "鼬ú",

  // 42
  42,

  // -42
  -42,

  // -0
  {
    type: "-0",
  },

  // Infinity
  {
    type: "Infinity",
  },

  // BigInt(1000000000000000000)
  {
    type: "BigInt",
    text: "1000000000000000000",
  },

  // 1n
  {
    type: "BigInt",
    text: "1",
  },

  // -2n
  {
    type: "BigInt",
    text: "-2",
  },

  // 0n
  {
    type: "BigInt",
    text: "0",
  },

  // ({})
  {
    type: "object",
    class: "Object",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // ({ foo: "bar"})
  {
    type: "object",
    class: "Object",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        foo: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "bar",
        },
      },
      ownPropertiesLength: 1,
    },
  },

  // []
  {
    type: "object",
    class: "Array",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "ArrayLike",
      items: [],
      length: 0,
    },
  },

  // [1]
  {
    type: "object",
    class: "Array",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "ArrayLike",
      items: [1],
      length: 1,
    },
  },

  // ["foo"]
  {
    type: "object",
    class: "Array",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "ArrayLike",
      items: ["foo"],
      length: 1,
    },
  },

  // new BigInt64Array()
  {
    type: "object",
    class: "BigInt64Array",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // const a = new BigInt64Array(1);
  // a[0] = BigInt(42);
  // a;
  //
  {
    type: "object",
    class: "BigInt64Array",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // new Map(
  //           Array.from({ length: 2 }).map((el, i) => [
  //             { key: i },
  //             { object: 42 },
  //           ])
  //         )
  {
    type: "object",
    class: "Map",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "MapLike",
      entries: [],
      size: 2,
    },
  },

  // new Map(Array.from({ length: 20 }).map((el, i) => [Symbol(i), i]))
  {
    type: "object",
    class: "Map",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "MapLike",
      entries: [],
      size: 20,
    },
  },

  // new Map(Array.from({ length: 331 }).map((el, i) => [Symbol(i), i]))
  {
    type: "object",
    class: "Map",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "MapLike",
      entries: [],
      size: 331,
    },
  },

  // new Set(Array.from({ length: 2 }).map((el, i) => ({ value: i })))
  {
    type: "object",
    class: "Set",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "ArrayLike",
      items: [
        {
          type: "object",
          class: "Object",
          ownPropertyLength: 0,
          isError: false,
          extensible: false,
          sealed: false,
          frozen: false,
          preview: {
            kind: "Object",
            ownProperties: {},
            ownPropertiesLength: 1,
          },
        },
        {
          type: "object",
          class: "Object",
          ownPropertyLength: 0,
          isError: false,
          extensible: false,
          sealed: false,
          frozen: false,
          preview: {
            kind: "Object",
            ownProperties: {},
            ownPropertiesLength: 1,
          },
        },
      ],
      length: 2,
    },
  },

  // new Set(Array.from({ length: 20 }).map((el, i) => i))
  {
    type: "object",
    class: "Set",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "ArrayLike",
      items: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15],
      length: 20,
    },
  },

  // new Set(Array.from({ length: 222 }).map((el, i) => i))
  {
    type: "object",
    class: "Set",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "ArrayLike",
      items: [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15],
      length: 222,
    },
  },

  // new Temporal.Instant(355924804000000000n)
  {
    type: "object",
    class: "Temporal.Instant",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // new Temporal.PlainDate(2021, 7, 1, "coptic")
  {
    type: "object",
    class: "Temporal.PlainDate",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // new Temporal.PlainDateTime(2021, 7, 1, 0, 0, 0, 0, 0, 0, "gregory")
  {
    type: "object",
    class: "Temporal.PlainDateTime",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // new Temporal.PlainMonthDay(7, 1, "chinese")
  {
    type: "object",
    class: "Temporal.PlainMonthDay",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // new Temporal.PlainTime(4, 20)
  {
    type: "object",
    class: "Temporal.PlainTime",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // new Temporal.PlainYearMonth(2021, 7, "indian")
  {
    type: "object",
    class: "Temporal.PlainYearMonth",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // new Temporal.ZonedDateTime(0n, "America/New_York")
  {
    type: "object",
    class: "Temporal.ZonedDateTime",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // Temporal.Duration.from({ years: 1 })
  {
    type: "object",
    class: "Temporal.Duration",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // myPolicy.createHTML("hello")
  {
    type: "object",
    class: "TrustedHTML",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
  },

  // myPolicy.createScript("const hello = 'world'")
  {
    type: "object",
    class: "TrustedScript",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
  },

  // myPolicy.createScriptURL("https://example.com/trusted")
  {
    type: "object",
    class: "TrustedScriptURL",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
  },

  // const formData = new FormData();
  // formData.append("a", 1);
  // formData.append("a", 2);
  // formData.append("b", 3);
  // formData;
  //
  {
    type: "object",
    class: "FormData",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
  },

  // customElements.define("fx-test", class extends HTMLElement {});
  // const { states } = document.createElement("fx-test").attachInternals();
  // states.add("custom-state");
  // states.add("another-custom-state");
  // states;
  //
  {
    type: "object",
    class: "CustomStateSet",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
  },

  // CSS.highlights.set("search", new Highlight());
  // CSS.highlights.set("glow", new Highlight());
  // CSS.highlights.set("anchor", new Highlight());
  // CSS.highlights;
  //
  {
    type: "object",
    class: "HighlightRegistry",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
  },

  // new URLSearchParams([
  //       ["a", 1],
  //       ["a", 2],
  //       ["b", 3],
  //       ["b", 3],
  //       ["b", 5],
  //       ["c", "this is 6"],
  //       ["d", 7],
  //       ["e", 8],
  //       ["f", 9],
  //       ["g", 10],
  //       ["h", 11],
  //     ])
  {
    type: "object",
    class: "URLSearchParams",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
  },

  // new Error("foo")
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "foo",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 1,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 1,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 4,
    },
  },

  // throw new Error("Long error ".repeat(10000));
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value:
            "Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long error Long e",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 7,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 1,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 4,
    },
  },

  // throw `“https://evil.com/?aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa“ is evil and “https://not-so-evil.com/?bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb“ is not good either`;
  //
  "“https://evil.com/?aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa“ is evil and “https://not-so-evil.com/?bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb“ is not good either",

  // Error("bar")
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "bar",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 6,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 1,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 4,
    },
  },

  // function bar() {
  //   asdf();
  // }
  // function foo() {
  //   bar();
  // }
  //
  // foo();
  //
  {
    type: "object",
    class: "ReferenceError",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "asdf is not defined",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 9,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 3,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 4,
    },
  },

  // eval("let a, a")
  {
    type: "object",
    class: "SyntaxError",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "redeclaration of let a",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 8,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 1,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 4,
    },
  },

  // throw "";
  "",

  // throw false;
  false,

  // throw undefined;
  {
    type: "undefined",
  },

  // throw 0;
  0,

  // throw { vegetable: "cucumber" };
  {
    type: "object",
    class: "Object",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        vegetable: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "cucumber",
        },
      },
      ownPropertiesLength: 1,
    },
  },

  // throw Symbol("potato");
  {
    type: "symbol",
    name: "potato",
  },

  // var err = new Error("pineapple");
  // err.name = "JuicyError";
  // err.flavor = "delicious";
  // throw err;
  //
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        flavor: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "delicious",
        },
        name: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "JuicyError",
        },
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "pineapple",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 17,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 2,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 6,
    },
  },

  // var originalError = new SyntaxError("original error");
  // var err = new Error("something went wrong", {
  //   cause: originalError,
  // });
  // throw err;
  //
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        cause: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: {
            type: "object",
            class: "SyntaxError",
            ownPropertyLength: 0,
            isError: false,
            extensible: false,
            sealed: false,
            frozen: false,
            preview: {
              kind: "Object",
              ownProperties: {},
              ownPropertiesLength: 4,
            },
          },
        },
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "something went wrong",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 17,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 3,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 5,
    },
  },

  // var a = new Error("err-a");
  // var b = new Error("err-b", { cause: a });
  // var c = new Error("err-c", { cause: b });
  // var d = new Error("err-d", { cause: c });
  // throw d;
  //
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        cause: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: {
            type: "object",
            class: "Error",
            ownPropertyLength: 0,
            isError: false,
            extensible: false,
            sealed: false,
            frozen: false,
            preview: {
              kind: "Object",
              ownProperties: {},
              ownPropertiesLength: 5,
            },
          },
        },
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "err-d",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 15,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 5,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 5,
    },
  },

  // var a = new Error("err-a", { cause: b });
  // var b = new Error("err-b", { cause: a });
  // throw b;
  //
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        cause: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: {
            type: "object",
            class: "Error",
            ownPropertyLength: 0,
            isError: false,
            extensible: false,
            sealed: false,
            frozen: false,
            preview: {
              kind: "Object",
              ownProperties: {},
              ownPropertiesLength: 5,
            },
          },
        },
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "err-b",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 15,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 3,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 5,
    },
  },

  // throw new Error("null cause", { cause: null });
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        cause: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: {
            type: "null",
          },
        },
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "null cause",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 7,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 1,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 5,
    },
  },

  // throw new Error("number cause", { cause: 0 });
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        cause: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 0,
        },
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "number cause",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 7,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 1,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 5,
    },
  },

  // throw new Error("string cause", { cause: "cause message" });
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        cause: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "cause message",
        },
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "string cause",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 7,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 1,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 5,
    },
  },

  // throw new Error("object cause", {
  //   cause: { code: 234, message: "ERR_234" },
  // });
  //
  {
    type: "object",
    class: "Error",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {
        cause: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: {
            type: "object",
            class: "Object",
            ownPropertyLength: 0,
            isError: false,
            extensible: false,
            sealed: false,
            frozen: false,
            preview: {
              kind: "Object",
              ownProperties: {},
              ownPropertiesLength: 2,
            },
          },
        },
        message: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "object cause",
        },
        columnNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 13,
        },
        lineNumber: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: 2,
        },
        fileName: {
          configurable: true,
          enumerable: true,
          writable: true,
          value: "eval line 7 > eval",
        },
      },
      ownPropertiesLength: 5,
    },
  },

  // Promise.reject("")
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // Promise.reject("tomato")
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // Promise.reject(false)
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // Promise.reject(0)
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // Promise.reject(null)
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // Promise.reject(undefined)
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // Promise.reject(Symbol("potato"))
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // Promise.reject({vegetable: "cucumber"})
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // Promise.reject(new Error("pumpkin"))
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // var err = new Error("pineapple");
  // err.name = "JuicyError";
  // err.flavor = "delicious";
  // Promise.reject(err);
  //
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },

  // Promise.resolve().then(() => {
  //         try {
  //           unknownFunc();
  //         } catch(e) {
  //           throw new Error("something went wrong", { cause: e })
  //         }
  //       })
  {
    type: "object",
    class: "Promise",
    ownPropertyLength: 0,
    isError: false,
    extensible: false,
    sealed: false,
    frozen: false,
    preview: {
      kind: "Object",
      ownProperties: {},
      ownPropertiesLength: 0,
    },
  },
];
